"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeSubslidesFromNormalizedSlides = exports.migrateSubslidesToChapters = exports.convertStoryboardStructureV1ToV2 = exports.getOrderedSlidesId = exports.restoreStoryboard = exports.restoreChaptersStructure = exports.restoreSlidesStructure = exports.getChapterType = exports.getParentSlide = exports.getRawSlideSubslides = exports.getSubslidesNestedData = exports.getSlideChapter = exports.getSlideType = void 0;
const path_1 = require("path");
const interfaces_1 = require("../types/interfaces");
const index_1 = require("./index");
function getSlideType(structure, slideId) {
    const archivedSlides = (0, index_1.getDefinitelyArchivedSlides)(structure);
    const hiddenSlides = (0, index_1.getHiddenSlides)(structure);
    const parentSlide = getParentSlide(structure, slideId);
    const slideChapter = getSlideChapter(structure, slideId) ?? '';
    const slideChapterType = getChapterType(structure, slideChapter);
    const slideChapterIsHidden = slideChapterType === interfaces_1.StructureApi.ChapterType.Hidden;
    const slideChapterIsArchived = slideChapterType === interfaces_1.StructureApi.ChapterType.Archived;
    const parentSlideIsHidden = parentSlide ? (0, index_1.listIncludesSlide)(hiddenSlides, parentSlide) : false;
    const slideIsDefinitelyHidden = (0, index_1.listIncludesSlide)(hiddenSlides, slideId);
    const parentSlideIsArchived = parentSlide ? (0, index_1.listIncludesSlide)(archivedSlides, parentSlide) : false;
    if (!slideChapter || slideChapterIsArchived || parentSlideIsArchived || (0, index_1.listIncludesSlide)(archivedSlides, slideId)) {
        return interfaces_1.StructureApi.SlideType.Archived;
    }
    else if (slideChapterIsHidden || parentSlideIsHidden || slideIsDefinitelyHidden) {
        return interfaces_1.StructureApi.SlideType.Hidden;
    }
    return interfaces_1.StructureApi.SlideType.Visible;
}
exports.getSlideType = getSlideType;
function getSlideChapter(structure, slideId) {
    const normalizedChapters = (0, index_1.getNormalizedChapters)(structure);
    const parentSlide = getParentSlide(structure, slideId);
    const chapterTuple = normalizedChapters.find(([, normalizedSlides]) => {
        return normalizedSlides.some(normalizedSlide => normalizedSlide.slide === (parentSlide ? parentSlide : slideId));
    });
    return chapterTuple?.[0];
}
exports.getSlideChapter = getSlideChapter;
function getSubslidesNestedData(structure, parentSlideId, systemSettings) {
    const subslides = getRawSlideSubslides(structure, parentSlideId);
    return subslides.map((rawSubslide) => {
        const subslideId = (0, index_1.normalizeSlide)(rawSubslide).slide;
        return {
            id: subslideId,
            name: structure.slides[subslideId].name,
            chapter: {
                id: getSlideChapter(structure, parentSlideId),
            },
            type: getSlideType(structure, subslideId),
            thumbnail: systemSettings ? (0, path_1.join)(systemSettings.path.slides, subslideId, systemSettings.path.slide.thumbnail) : undefined,
            template: structure.slides[subslideId].template,
        };
    });
}
exports.getSubslidesNestedData = getSubslidesNestedData;
function getRawSlideSubslides(structure, slideId) {
    const chapterName = getSlideChapter(structure, slideId);
    if (chapterName) {
        const normalChapters = (0, index_1.getNormalizedChapters)(structure);
        const searchedSlide = normalChapters.flatMap(([, slides]) => slides)
            .find(normalSlide => normalSlide.slide === slideId);
        return searchedSlide?.subslides || [];
    }
    return [];
}
exports.getRawSlideSubslides = getRawSlideSubslides;
function getParentSlide(structure, slideId) {
    const searchedSlide = (0, index_1.getSlidesWithSubslides)(structure)
        .find(slide => {
        return slide.subslides?.some(subslide => {
            return (0, index_1.normalizeSlide)(subslide).slide === slideId;
        });
    });
    return searchedSlide?.slide;
}
exports.getParentSlide = getParentSlide;
function getChapterType(structure, chapterId) {
    const normalizedStoryboard = structure.storyboard.map(chapterEntity => (0, index_1.normalizeChapter)(chapterEntity));
    const chapterInStoryBoard = normalizedStoryboard.find(normalizedChapter => normalizedChapter.chapter === chapterId);
    if (!chapterInStoryBoard) {
        return interfaces_1.StructureApi.ChapterType.Archived;
    }
    return chapterInStoryBoard.hidden ? interfaces_1.StructureApi.ChapterType.Hidden : interfaces_1.StructureApi.ChapterType.Visible;
}
exports.getChapterType = getChapterType;
function restoreSlidesStructure(nestedSlidesData, structureScheme) {
    const chaptersList = [];
    nestedSlidesData.forEach(nestedSlide => {
        const slideData = {
            name: nestedSlide.name,
            template: nestedSlide.template,
            ...nestedSlide?.custom
        };
        const slideSubslides = nestedSlide.subslides.map(subslide => {
            if (subslide.type === interfaces_1.StructureApi.SlideType.Hidden) {
                return `!${subslide.id}`;
            }
            return subslide.id;
        });
        if (slideSubslides.length) {
            slideData.subslides = slideSubslides;
        }
        structureScheme.slides[nestedSlide.id] = slideData;
        if (nestedSlide.chapter?.id && !chaptersList.some(chapterName => chapterName === nestedSlide.chapter?.id)) {
            chaptersList.push(nestedSlide.chapter.id);
        }
    });
    return chaptersList;
}
exports.restoreSlidesStructure = restoreSlidesStructure;
function restoreChaptersStructure(nestedSlidesData, structureScheme) {
    const subslidesId = nestedSlidesData.filter(nestedSlide => nestedSlide.subslides.length).flatMap(nestedSlide => nestedSlide.subslides.map(subslide => subslide.id));
    const nestedSlidesWithoutSubslides = nestedSlidesData.filter(nestedSlide => !subslidesId.includes(nestedSlide.id));
    const nestedChapters = [];
    nestedSlidesWithoutSubslides.forEach(nestedSlide => {
        if (nestedSlide.chapter && !nestedChapters.map(slideData => slideData.id).includes(nestedSlide.chapter?.id)) {
            nestedChapters.push(nestedSlide.chapter);
        }
    });
    nestedChapters.forEach(nestedChapter => {
        const currentChapterContent = (0, index_1.getChapterSlidesFromNested)(nestedSlidesWithoutSubslides, nestedChapter);
        if (nestedChapter.id) {
            structureScheme.chapters[nestedChapter.id] = {
                name: nestedChapter.name,
                content: currentChapterContent,
                ...(0, index_1.getChapterCustomFields)(structureScheme, nestedChapter.id)
            };
        }
    });
    return nestedChapters;
}
exports.restoreChaptersStructure = restoreChaptersStructure;
function restoreStoryboard(nestedChapters, structureScheme) {
    nestedChapters.forEach((chapter) => {
        const normalizedChapter = chapter.type === interfaces_1.StructureApi.ChapterType.Hidden ? `!${chapter.id}` : chapter.id;
        if (normalizedChapter && typeof chapter.storyboardIndex === 'number' && chapter.storyboardIndex >= 0) {
            structureScheme.storyboard[chapter.storyboardIndex] = normalizedChapter;
        }
    });
}
exports.restoreStoryboard = restoreStoryboard;
function getOrderedSlidesId(allChaptersId, structureJson) {
    const slidesId = allChaptersId.flatMap(chapterId => {
        const chapterSlides = structureJson.chapters[chapterId].content;
        const flattedChapterSlides = (0, index_1.flatChapterContent)(chapterSlides);
        return flattedChapterSlides
            .map(flatedSlide => (0, index_1.normalizeSlide)(flatedSlide).slide);
    });
    if (structureJson.archivedSlides) {
        slidesId.push(...structureJson.archivedSlides.map(archivedSlide => archivedSlide.slide));
    }
    return slidesId;
}
exports.getOrderedSlidesId = getOrderedSlidesId;
function convertStoryboardStructureV1ToV2(storyboard) {
    return storyboard.map(chapterName => (0, index_1.getNormalizedChapterEntity)(chapterName));
}
exports.convertStoryboardStructureV1ToV2 = convertStoryboardStructureV1ToV2;
function migrateSubslidesToChapters(normalizedSlidesWithSubslides, chapters, structureJson) {
    return normalizedSlidesWithSubslides.map(slideEntry => {
        const slideName = slideEntry[0];
        const slideData = slideEntry[1];
        const chapterName = getSlideChapter(structureJson, slideName);
        const chapterContent = chapters[chapterName].content;
        const slideEntityIndex = chapterContent.findIndex(slideEntity => {
            if (typeof slideEntity === 'string') {
                return -1;
            }
            slideEntity.slide === slideName;
        });
        if (chapterContent.includes(slideName)) {
            const index = chapterContent.findIndex(slide => slide === slideName);
            const normalizedSlideData = {
                slide: slideName,
                subslides: slideData.subslides
            };
            chapterContent[index] = normalizedSlideData;
        }
        else if (slideEntityIndex >= 0) {
            const chapterSlide = chapterContent[slideEntityIndex];
            chapterSlide.subslides = slideData.subslides;
        }
        return chapters;
    })[0];
}
exports.migrateSubslidesToChapters = migrateSubslidesToChapters;
function removeSubslidesFromNormalizedSlides(slides) {
    return Object.fromEntries(Object.entries(slides).map(entry => {
        const { name, template } = entry[1];
        return [entry[0], { name, template }];
    }));
}
exports.removeSubslidesFromNormalizedSlides = removeSubslidesFromNormalizedSlides;
//# sourceMappingURL=helpers.js.map